"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.File = void 0;
const assert_1 = __importDefault(require("assert"));
const fs_1 = require("fs");
const mkdirp_1 = __importDefault(require("mkdirp"));
const path_1 = require("path");
const resolve_1 = __importDefault(require("resolve"));
const Package_1 = require("./Package");
const replaceImports_1 = require("./utils/replaceImports");
class File {
    constructor(opts) {
        this.pkg = opts.pkg;
        this.filePath = opts.filePath;
        this.absFilePath = path_1.join(this.pkg.pkgRoot, this.filePath);
        this.absFileDir = path_1.dirname(this.absFilePath);
        this.parse();
    }
    writeContent(content) {
        const filePath = path_1.join(this.pkg.typesRoot, this.filePath);
        mkdirp_1.default.sync(path_1.dirname(filePath));
        fs_1.writeFileSync(filePath, content, 'utf-8');
    }
    getPrefix({ isExternal }) {
        let filePath = this.filePath;
        // 删除前置的 ./
        filePath = filePath.replace(/^\.\//, '');
        // 计算路径长度，非 root +1
        let len = filePath.split('/').length;
        const isRoot = this.pkg === this.pkg.rootPackage;
        if (!isRoot) {
            len += 1;
        }
        // external 时要往上再找一级
        if (isExternal) {
            len += 1;
        }
        // 返回 ./ 或多个 ../
        if (len === 1) {
            return './';
        }
        else {
            return new Array(len).join('../');
        }
    }
    getExternals() {
        const externals = this.pkg.opts.externals || [];
        return externals.reduce((memo, key) => {
            memo[key] = `${this.getPrefix({ isExternal: true })}${key}`;
            return memo;
        }, {});
    }
    parse() {
        console.log('> ', this.filePath, this.absFilePath);
        const { content, localImports, depImports } = replaceImports_1.replaceImports({
            content: fs_1.readFileSync(this.absFilePath, 'utf-8'),
            prefix: this.getPrefix({}),
            externals: this.getExternals(),
        });
        this.writeContent(content);
        localImports.forEach((localImport) => {
            console.log('>> local import', localImport);
            const filePath = path_1.relative(this.pkg.pkgRoot, resolve_1.default.sync(localImport, {
                basedir: path_1.dirname(this.absFilePath),
                extensions: ['.d.ts'],
            }));
            if (!this.pkg.files[filePath]) {
                this.pkg.addFile(filePath);
                new File({
                    pkg: this.pkg,
                    filePath,
                });
            }
        });
        depImports.forEach((depImport) => {
            console.log('>> dep import', depImport);
            const len = depImport.split('/').length;
            const scoped = depImport.charAt(0) === '@';
            if (scoped) {
                assert_1.default(len === 2, `暂不支持 deepImport, ${depImport}`);
            }
            else {
                assert_1.default(len === 1, `暂不支持 deepImport, ${depImport}`);
            }
            if (!this.pkg.rootPackage.packages[depImport]) {
                this.pkg.rootPackage.packages[depImport] = true;
                new Package_1.Package({
                    cwd: this.absFileDir,
                    name: depImport,
                    typesRoot: path_1.join(this.pkg.rootPackage.typesRoot, depImport),
                    rootPackage: this.pkg.rootPackage,
                });
            }
        });
    }
}
exports.File = File;
